@@ -8,7 +8,7 @@
 ;; Author: Jonas Bernoulli <jonas@bernoul.li>
 ;; Maintainer: Jonas Bernoulli <jonas@bernoul.li>
 
-;; Package-Requires: ((emacs "24.4") (async "1.9") (dash "2.13.0"))
+;; Package-Requires: ((emacs "24.4") (async "1.9"))
 ;; Keywords: tools
 ;; Homepage: https://github.com/magit/with-editor
 
@@ -80,10 +80,8 @@
 ;;; Code:
 
 (require 'cl-lib)
-(require 'dash)
 (require 'server)
-(require 'tramp)
-(require 'tramp-sh nil t)
+(require 'shell)
 
 (and (require 'async-bytecomp nil t)
      (memq 'magit (bound-and-true-p async-bytecomp-allowed-packages))
@@ -108,21 +106,22 @@
 
 (defun with-editor-locate-emacsclient ()
   "Search for a suitable Emacsclient executable."
-  (--if-let (with-editor-locate-emacsclient-1 (with-editor-emacsclient-path) 3)
-      it
-    (display-warning 'with-editor (format "\
+  (or (with-editor-locate-emacsclient-1
+       (with-editor-emacsclient-path)
+       (length (split-string emacs-version "\\.")))
+      (prog1 nil (display-warning 'with-editor "\
 Cannot determine a suitable Emacsclient
 
 Determining an Emacsclient executable suitable for the
 current Emacs instance failed.  For more information
-please see https://github.com/magit/magit/wiki/Emacsclient."))
-    nil))
+please see https://github.com/magit/magit/wiki/Emacsclient."))))
 
 (defun with-editor-locate-emacsclient-1 (path depth)
-  (let* ((version-lst (-take depth (split-string emacs-version "\\.")))
+  (let* ((version-lst (cl-subseq (split-string emacs-version "\\.") 0 depth))
          (version-reg (concat "^" (mapconcat #'identity version-lst "\\."))))
     (or (locate-file-internal
-         "emacsclient" path
+         (if (equal invocation-name "remacs") "remacsclient" "emacsclient")
+         path
          (cl-mapcan
           (lambda (v) (cl-mapcar (lambda (e) (concat v e)) exec-suffixes))
           (nconc (and (boundp 'debian-emacs-flavor)
@@ -140,8 +139,9 @@ please see https://github.com/magit/magit/wiki/Emacsclient."))
              (with-editor-locate-emacsclient-1 path (1- depth))))))
 
 (defun with-editor-emacsclient-version (exec)
-  (-when-let (1st-line (car (process-lines exec "--version")))
-    (cadr (split-string 1st-line))))
+  (let ((default-directory (file-name-directory exec)))
+    (ignore-errors
+      (cadr (split-string (car (process-lines exec "--version")))))))
 
 (defun with-editor-emacsclient-path ()
   (let ((path exec-path))
@@ -176,10 +176,10 @@ trap \"kill $sleep; exit 1\" USR2; \
 wait $sleep'"
   "The sleeping editor, used when the Emacsclient cannot be used.
 
-This fallback is used for asynchronous process started inside the
-macro `with-editor', when the process runs on a remote machine or
-for local processes when `with-editor-emacsclient-executable' is
-nil (i.e. when no suitable Emacsclient was found, or the user
+This fallback is used for asynchronous processes started inside
+the macro `with-editor', when the process runs on a remote machine
+or for local processes when `with-editor-emacsclient-executable'
+is nil (i.e. when no suitable Emacsclient was found, or the user
 decided not to use it).
 
 Where the latter uses a socket to communicate with Emacs' server,
@@ -255,7 +255,7 @@ has a reason to disrespect `server-window' and it likely is
 not a good idea to change such entries.")
 
 (defvar with-editor-file-name-history-exclude nil
-  "List of regexps for filenames `server-visit' should no remember.
+  "List of regexps for filenames `server-visit' should not remember.
 When a filename matches any of the regexps, then `server-visit'
 does not add it to the variable `file-name-history', which is
 used when reading a filename in the minibuffer.")
@@ -329,15 +329,17 @@ used when reading a filename in the minibuffer.")
                  (ignore-errors
                    (server-send-string client "-error Canceled by user"))
                  (delete-process client))
-             ;; Fallback for when emacs was used as $EDITOR instead
-             ;; of emacsclient or the sleeping editor.  See #2258.
+             ;; Fallback for when emacs was used as $EDITOR
+             ;; instead of emacsclient or the sleeping editor.
+             ;; See https://github.com/magit/magit/issues/2258.
              (ignore-errors (delete-file buffer-file-name))
              (kill-buffer)))
           (t
            (save-buffer)
            (if clients
-               ;; Don't use `server-edit' because we do not want to show
-               ;; another buffer belonging to another client.  See #2197.
+               ;; Don't use `server-edit' because we do not want to
+               ;; show another buffer belonging to another client.
+               ;; See https://github.com/magit/magit/issues/2197.
                (server-done)
              (kill-buffer))))
     (when pid
@@ -370,7 +372,7 @@ used when reading a filename in the minibuffer.")
   ;; either `with-editor-finish' or `with-editor-cancel',
   ;; and from removing the key bindings for these commands.
   (unless with-editor-mode
-    (error "With-Editor mode cannot be turned off"))
+    (user-error "With-Editor mode cannot be turned off"))
   (add-hook 'kill-buffer-query-functions
             'with-editor-kill-buffer-noop nil t)
   ;; `server-execute' displays a message which is not
@@ -381,7 +383,7 @@ used when reading a filename in the minibuffer.")
 (put 'with-editor-mode 'permanent-local t)
 
 (defun with-editor-kill-buffer-noop ()
-  (message (substitute-command-keys "\
+  (user-error (substitute-command-keys "\
 Don't kill this buffer.  Instead cancel using \\[with-editor-cancel]")))
 
 (defun with-editor-usage-message ()
@@ -409,43 +411,47 @@ ENVVAR is provided then bind that environment variable instead.
                                    (pop body)
                                  '(or with-editor--envvar "EDITOR")))
          (process-environment process-environment))
-     (if (or (not with-editor-emacsclient-executable)
-             (file-remote-p default-directory))
-         (push (concat with-editor--envvar "=" with-editor-sleeping-editor)
-               process-environment)
-       ;; Make sure server-use-tcp's value is valid.
-       (unless (featurep 'make-network-process '(:family local))
-         (setq server-use-tcp t))
-       ;; Make sure the server is running.
-       (unless server-process
-         (when (server-running-p server-name)
-           (setq server-name (format "server%s" (emacs-pid)))
-           (when (server-running-p server-name)
-             (server-force-delete server-name)))
-         (server-start))
-       ;; Tell $EDITOR to use the Emacsclient.
-       (push (concat with-editor--envvar "="
-                     (shell-quote-argument with-editor-emacsclient-executable)
-       ;; Tell the process where the server file is.
-                     (and (not server-use-tcp)
-                          (concat " --socket-name="
-                                  (shell-quote-argument
-                                   (expand-file-name server-name
-                                                     server-socket-dir)))))
-             process-environment)
-       (when server-use-tcp
-         (push (concat "EMACS_SERVER_FILE="
-                       (expand-file-name server-name server-auth-dir))
-               process-environment))
-       ;; As last resort fallback to the sleeping editor.
-       (push (concat "ALTERNATE_EDITOR=" with-editor-sleeping-editor)
-             process-environment))
+     (with-editor--setup)
      ,@body))
 
+(defun with-editor--setup ()
+  (if (or (not with-editor-emacsclient-executable)
+          (file-remote-p default-directory))
+      (push (concat with-editor--envvar "=" with-editor-sleeping-editor)
+            process-environment)
+    ;; Make sure server-use-tcp's value is valid.
+    (unless (featurep 'make-network-process '(:family local))
+      (setq server-use-tcp t))
+    ;; Make sure the server is running.
+    (unless server-process
+      (when (server-running-p server-name)
+        (setq server-name (format "server%s" (emacs-pid)))
+        (when (server-running-p server-name)
+          (server-force-delete server-name)))
+      (server-start))
+    ;; Tell $EDITOR to use the Emacsclient.
+    (push (concat with-editor--envvar "="
+                  (shell-quote-argument with-editor-emacsclient-executable)
+                  ;; Tell the process where the server file is.
+                  (and (not server-use-tcp)
+                       (concat " --socket-name="
+                               (shell-quote-argument
+                                (expand-file-name server-name
+                                                  server-socket-dir)))))
+          process-environment)
+    (when server-use-tcp
+      (push (concat "EMACS_SERVER_FILE="
+                    (expand-file-name server-name server-auth-dir))
+            process-environment))
+    ;; As last resort fallback to the sleeping editor.
+    (push (concat "ALTERNATE_EDITOR=" with-editor-sleeping-editor)
+          process-environment)))
+  
 (defun with-editor-server-window ()
   (or (and buffer-file-name
-           (cdr (--first (string-match-p (car it) buffer-file-name)
-                         with-editor-server-window-alist)))
+           (cdr (cl-find-if (lambda (cons)
+                              (string-match-p (car cons) buffer-file-name))
+                            with-editor-server-window-alist)))
       server-window))
 
 (defun server-switch-buffer--with-editor-server-window-alist
@@ -516,10 +522,7 @@ which may or may not insert the text into the PROCESS' buffer."
           (with-current-buffer
               (find-file-noselect
                (if (file-name-absolute-p file)
-                   (if (tramp-tramp-file-p default-directory)
-                       (with-parsed-tramp-file-name default-directory nil
-                         (tramp-make-tramp-file-name method user host file hop))
-                     file)
+                   (concat (file-remote-p default-directory) file)
                  (expand-file-name file)))
             (with-editor-mode 1)
             (setq with-editor--pid pid)
@@ -545,8 +548,9 @@ which may or may not insert the text into the PROCESS' buffer."
     (files _proc &optional _nowait)
   (dolist (file files)
     (setq  file (car file))
-    (when (--any (string-match-p it file)
-                 with-editor-file-name-history-exclude)
+    (when (cl-find-if (lambda (regexp)
+                        (string-match-p regexp file))
+                      with-editor-file-name-history-exclude)
       (setq file-name-history (delete file file-name-history)))))
 
 ;;; Augmentations
@@ -662,10 +666,10 @@ else like the former."
 (defun with-editor-shell-command-read-args (prompt &optional async)
   (let ((command (read-shell-command
                   prompt nil nil
-                  (--when-let (or buffer-file-name
-                                  (and (eq major-mode 'dired-mode)
-                                       (dired-get-filename nil t)))
-                    (file-relative-name it)))))
+                  (let ((filename (or buffer-file-name
+                                      (and (eq major-mode 'dired-mode)
+                                           (dired-get-filename nil t)))))
+                    (and filename (file-relative-name filename))))))
     (list command
           (if (or async (setq async (string-match-p "&[ \t]*\\'" command)))
               (< (prefix-numeric-value current-prefix-arg) 0)
@@ -733,14 +737,14 @@ See info node `(with-editor)Debugging' for instructions."
      (format "  server-name: %s\n" server-name)
      (format "  server-socket-dir: %s\n" server-socket-dir))
     (if (and server-socket-dir (file-accessible-directory-p server-socket-dir))
-        (--each (directory-files server-socket-dir nil "^[^.]")
-          (insert (format "    %s\n" it)))
+        (dolist (file (directory-files server-socket-dir nil "^[^.]"))
+          (insert (format "    %s\n" file)))
       (insert (format "    %s: not an accessible directory\n"
                       (if server-use-tcp "WARNING" "ERROR"))))
     (insert (format "  server-auth-dir: %s\n" server-auth-dir))
     (if (file-accessible-directory-p server-auth-dir)
-        (--each (directory-files server-auth-dir nil "^[^.]")
-          (insert (format "    %s\n" it)))
+        (dolist (file (directory-files server-auth-dir nil "^[^.]"))
+          (insert (format "    %s\n" file)))
       (insert (format "    %s: not an accessible directory\n"
                       (if server-use-tcp "ERROR" "WARNING"))))
     (let ((val with-editor-emacsclient-executable)
@@ -759,11 +763,11 @@ See info node `(with-editor)Debugging' for instructions."
             (format "  $PATH: %S\n" (getenv "PATH"))
             (format "  exec-path: %s\n" exec-path))
     (insert (format "  with-editor-emacsclient-path:\n"))
-    (--each (with-editor-emacsclient-path)
-      (insert (format "    %s (%s)\n" it (car (file-attributes it))))
-      (when (file-directory-p it)
+    (dolist (dir (with-editor-emacsclient-path))
+      (insert (format "    %s (%s)\n" dir (car (file-attributes dir))))
+      (when (file-directory-p dir)
         ;; Don't match emacsclientw.exe, it makes popup windows.
-        (dolist (exec (directory-files it t "emacsclient\\(?:[^w]\\|\\'\\)"))
+        (dolist (exec (directory-files dir t "emacsclient\\(?:[^w]\\|\\'\\)"))
           (insert (format "      %s (%s)\n" exec
                           (with-editor-emacsclient-version exec))))))))
 
